/*
 *
 * JONAMA - Secure TCP Relay with SSLeay
 *
 * ==================================================================== 
 * Copyright (c) 1999 Henri Gomez. All rights reserved. 
 *
 * --------------------------------------------------------------------
 *
 * "This product includes software developed by 
 * Ralf S. Engelschall <rse@engelschall.com> for use in the
 * mod_ssl project (http://www.engelschall.com/sw/mod_ssl/)." 
 *
 * --------------------------------------------------------------------
 */


#include <stdio.h>
#include <stdlib.h>
#include <ssl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <resolv.h>
#include <evp.h>
#include <err.h>
#include <stdarg.h>
#include <sys/types.h>
#include <dirent.h>
#include <netdb.h> 
#include <pwd.h>
#include <grp.h> 
#include <signal.h>
#include <ctype.h>  
 

/*
 * Provide reasonable default for some defines
 */
#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE  !FALSE
#endif
#ifndef UNSET
#define UNSET -1
#endif
#ifndef NUL
#define NUL   '\0'
#endif

#ifdef OPENSSL_VERSION_NUMBER
#define SSL_LIBRARY_VERSION OPENSSL_VERSION_NUMBER
#define SSL_LIBRARY_NAME    "OpenSSL"
#else
#ifdef SSLEAY_VERSION_NUMBER
#define SSL_LIBRARY_VERSION SSLEAY_VERSION_NUMBER
#define SSL_LIBRARY_NAME    "SSLeay"
#else
#define SSL_LIBRARY_VERSION 0x0000
#define SSL_LIBRARY_NAME    "OtherSSL"
#endif
#endif


/*
 * Define the SSL options
 */
#define SSL_OPT_NONE           (0)
#define SSL_OPT_RELSET         (1<<0)
#define SSL_OPT_COMPATENVVARS  (1<<1)
#define SSL_OPT_EXPORTCERTDATA (1<<2)
#define SSL_OPT_FAKEBASICAUTH  (1<<3)
#define SSL_OPT_ALL            (SSL_OPT_COMPATENVVAR|SSL_OPT_EXPORTCERTDATA|SSL_OPT_FAKEBASICAUTH)
typedef int OptionType;


/*
 * Define the SSL Protocol options
 */

#define SSL_PROTOCOL_NONE  (0)
#define SSL_PROTOCOL_SSLV2 (1<<0)
#define SSL_PROTOCOL_SSLV3 (1<<1)
#define SSL_PROTOCOL_TLSV1 (1<<2)
#define SSL_PROTOCOL_ALL   (SSL_PROTOCOL_SSLV2|SSL_PROTOCOL_SSLV3|SSL_PROTOCOL_TLSV1)
typedef int ProtocolType;

/*
 * Define the SSL verify levels
 */
typedef enum
{
    SSL_CVERIFY_UNSET           = UNSET,
    SSL_CVERIFY_NONE            = 0,
    SSL_CVERIFY_OPTIONAL        = 1,
    SSL_CVERIFY_REQUIRE         = 2,
    SSL_CVERIFY_OPTIONAL_NO_CA  = 3
} VerifyType;

typedef enum
{
    C_MODE_UNSET		= UNSET,
    C_MODE_NORMAL		= 0,
    C_MODE_BALANCE		= 1
}
ConnectType;

typedef enum 
{
	T_UNSET		= UNSET,
	T_MASTER	= 0,
	T_SERVER	= 1,
	T_CLIENT
}
ThreadMode;


#define	MAX_CONNECTS		32
#define	MAX_JONAMA		32

#define	JONAMA_CONF_FILE	"/etc/jonama/conf/jonama.conf"

#define JONAMA_DEF_CERT_PATH	"/etc/jonama/conf/ssl.crt/cacert.pem"
#define JONAMA_DEF_KEY_PATH	"/etc/jonama/conf/ssl.key/cakey.pem"

#define	S_BUFF_SIZE		2048

typedef struct remoteInfo
{
	char *			hostname;
	struct in_addr		hostaddr;
	int			port;
}
remoteInfo;

typedef struct jonama
{
        SSL_CTX *       context;
        SSL *           connexion;

	char *		name;

        int             protocolLevel;
        int             verifyDepth;

        VerifyType	verifyMode;
	char *		certificateFile;
	char *		privateKeyFile;
	char *		CAPath;
	char *		CACertificateFile;
	char *		cyphers;

	int		acceptPort;
	remoteInfo	connectTo[MAX_CONNECTS];
	int		numConnects;
	ConnectType	connectMode;

	int		bufferSize;
	int		resolveRemote;
}
jonama;

#define	LOG_MINI		0
#define	LOG_NORMAL		1
#define	LOG_ALLWAYS		2

#ifndef INADDR_LOOPBACK
#define	INADDR_LOOPBACK		((u_long)0x7F000001)
#endif

#include "jonama_vars.h"
#include "jonama_protos.h"
