/*
 *
 * JONAMA - Secure TCP Relay with SSLeay
 *
 * ====================================================================
 * Copyright (c) 1999 Henri Gomez. All rights reserved.
 *
 * --------------------------------------------------------------------
 *
 * "This product includes software developed by
 * Ralf S. Engelschall <rse@engelschall.com> for use in the
 * mod_ssl project (http://www.engelschall.com/sw/mod_ssl/)."
 *
 * --------------------------------------------------------------------
 */

#include "jonama.h"

static jonama *	curwarp;
static int	inlocal = 0;


static uid_t uname2id(const char *name)
{
	struct passwd *ent;

	if (name[0] == '#')
        	return (atoi(&name[1]));

	if (!(ent = getpwnam(name)))
	{
        	fprintf(stderr, "bad user name %s\n", name);
        	exit(1);
    	}

	return (ent->pw_uid);
}


gid_t gname2id(const char *name)
{
	struct group *ent;

	if (name[0] == '#')
		return (atoi(&name[1]));

	if (!(ent = getgrnam(name)))
	{
        	fprintf(stderr, "bad group name %s\n", name);
        	exit(1);
	}

	return (ent->gr_gid);
}


static char * allocString(char * orgstr)
{
	char *	ptr;

	ptr = malloc(strlen(orgstr) + 1);

	if (ptr == NULL)
	{
		fprintf(stderr, "Can't allocate mem for str");
		exit(1);
	}

	return (ptr);
}


static char * copyString(char * orgstr)
{
	char * ptr;

	ptr = allocString(orgstr);
	strcpy(ptr, orgstr);
	return (ptr);
}


static int setString(char ** dststr, char * orgstr)
{
	if (*dststr)
		free(*dststr);

	*dststr = copyString(orgstr);
	return(strlen(orgstr));
}


static int getTokens(char * line, char * tokens[], int maxtokens)
{
		int	i;
		int	pos = 0;
		int	len = strlen(line);
		int	got = 0;

	
	for (i = 0; i < len; i++, line++)
	{
		if (*line <= ' ')
		{
			got = 0;
			*line = 0;
		}
		else
		{
			if (! got)
			{
				tokens[pos] = line;
				pos++;
				got = 1;
			}
		}
	}

/*
	for (i = 0; i < pos; i++)
		printf("%s %d\n", tokens[i], strlen(tokens[i]));
*/

	return (pos);
}


static int buildRemoteInfo(char * str, remoteInfo * rinfo)
{
	struct hostent *	connectent;
	char *			ptr;
	char 			hstr[512];
	int			hlen;


	ptr = rindex(str, ':');

	if (str[0] == ':')
		strcpy(hstr, "localhost");
	else
	{
		hlen = ptr - str;
                strncpy(hstr, str, hlen);
                hstr[hlen] = 0;
	} 			
		
	if ((connectent = gethostbyname(hstr)) == NULL)	
	{
		ja_logoserr(LOG_ALLWAYS, "Can't resolv %s", hstr);
		return (-1);
	}

	setString(&rinfo->hostname, hstr);
	memcpy(&rinfo->hostaddr, connectent->h_addr_list[0], sizeof(struct in_addr));
	rinfo->port = atoi(&ptr[1]);	
	
	return (1);
}

static int processLine(char * line, int lnum)
{
	char *		fields[32];
	int		nfields;
	char *		ptr;


	nfields = getTokens(line, fields, 32);

	if (nfields < 1)
		return(0);

	if (fields[0][0] == '#')
		return (0);
/*
 * Look for global vars
 */

/*
 *	printf("[%s] %d [%s] %d\n", fields[0], strlen(fields[0]), fields[1], strlen(fields[1]));
 */
	if (! inlocal)
	{
		if (nfields < 2)
			return (0);

		if (! strcasecmp(fields[0], "rootpath"))
			return(setString(&chrootPath, fields[1]));
	
		if (! strcasecmp(fields[0], "logpath"))
			return(setString(&logPath, fields[1]));

		if (! strcasecmp(fields[0], "logfile"))
			return(setString(&logFileName, fields[1]));

		if (! strcasecmp(fields[0], "errorlogfile"))
			return(setString(&errorFileName, fields[1]));

		if (! strcasecmp(fields[0], "user"))
		{
			user = uname2id(fields[1]);
			return(1);
		}

		if (! strcasecmp(fields[0], "group"))
		{
			group = gname2id(fields[1]);
			return(1);
		}

		if (! strcasecmp(fields[1], "{"))
		{
			inlocal = 1;
			curwarp = &ja_pool[numJa++];
			
			curwarp->name = copyString(fields[0]);
			return(1);
		}
			
	}
	else
	{
		if (! strcasecmp(fields[0], "}"))
		{
			inlocal = 0;

			if (curwarp->certificateFile == NULL)
				curwarp->certificateFile = JONAMA_DEF_CERT_PATH;

			if (curwarp->privateKeyFile == NULL)
				curwarp->privateKeyFile = JONAMA_DEF_KEY_PATH;

			return (1);
		}

		if (nfields < 2)
			return (0);

		if (! strcasecmp(fields[0], "certificatefile"))
			return(setString(&curwarp->certificateFile, fields[1]));

		if (! strcasecmp(fields[0], "privatekeyfile"))
			return(setString(&curwarp->privateKeyFile, fields[1]));

		if (! strcasecmp(fields[0], "capath"))
			return(setString(&curwarp->CAPath, fields[1]));

		if (! strcasecmp(fields[0], "cafile"))
			return(setString(&curwarp->CACertificateFile, fields[1]));

		if (! strcasecmp(fields[0], "cyphers"))
			return(setString(&curwarp->cyphers, fields[1]));
	
		if (! strcasecmp(fields[0], "verifydepth"))
		{
			curwarp->verifyDepth = atoi(fields[1]);
			return (1);
		}

		if (! strcasecmp(fields[0], "verifymode"))
		{
			if (! strcasecmp(fields[1], "none"))
				curwarp->verifyMode = SSL_CVERIFY_NONE;
			else if (! strcasecmp(fields[1], "require"))
				curwarp->verifyMode = SSL_CVERIFY_REQUIRE;
			else if (! strcasecmp(fields[1], "optional"))
				curwarp->verifyMode = SSL_CVERIFY_OPTIONAL;
			else if (! strcasecmp(fields[1], "optionalnoca"))
				curwarp->verifyMode = SSL_CVERIFY_OPTIONAL_NO_CA;
			else
			{
				fprintf(stderr, "unknown verify mode %s at line %d\n", fields[1], lnum);
				return (0);
			}	
		}

		if (! strcasecmp(fields[0], "connectto"))
		{
			if (curwarp->numConnects == MAX_CONNECTS)
			{
				fprintf(stderr, "maxConnects (%d) allready reached", MAX_CONNECTS);
				return (0);
			}

			if ((ptr = rindex(fields[1], ':')) == NULL)
			{
				fprintf(stderr, "connectto: form is host:port, got %s at line %d\n", fields[1], lnum);
				return (0);
			}

			if (atoi(&ptr[1]) <= 0)
			{
				fprintf(stderr, "connectto: port must be numeric and was (%s) at line %d\n", &ptr[1], lnum);
				return(0);
			}

			if (buildRemoteInfo(fields[1], &curwarp->connectTo[curwarp->numConnects]) > 0)
				curwarp->numConnects++;

			return (1);
		}

		if (! strcasecmp(fields[0], "acceptport"))
		{
			curwarp->acceptPort = atoi(fields[1]);
			return (1);
		}

		if (! strcasecmp(fields[0], "buffersize"))
		{
			curwarp->bufferSize = atoi(fields[1]);
			return (1);
		}

		if (! strcasecmp(fields[0], "resolve"))
		{
			curwarp->resolveRemote = atoi(fields[1]);;
			return (1);
		}

		if (! strcasecmp(fields[0], "verifymode"))
		{
			if (! strcasecmp(fields[1], "none"))
				curwarp->verifyMode = SSL_CVERIFY_NONE;
			else if (! strcasecmp(fields[1], "require"))
				curwarp->verifyMode = SSL_CVERIFY_REQUIRE;
			else if (! strcasecmp(fields[1], "optional"))
				curwarp->verifyMode = SSL_CVERIFY_OPTIONAL;
			else if (! strcasecmp(fields[1], "optionalnoca"))
				curwarp->verifyMode = SSL_CVERIFY_OPTIONAL_NO_CA;
			else
			{
				fprintf(stderr, "unknown verifyMode %s at line %d\n", fields[1], lnum);
				return (0);
			}	

			return (1);
		}

		if (! strcasecmp(fields[0], "connectmode"))
		{
			if (! strcasecmp(fields[1], "normal"))
				curwarp->connectMode = C_MODE_NORMAL;
			else if (! strcasecmp(fields[1], "balance"))
				curwarp->connectMode = C_MODE_BALANCE;
			else
			{
				fprintf(stderr, "unknown connectMode %s at line %d\n", fields[1], lnum);
				return (0);
			}

			return (1);
		}
	}

	fprintf(stderr, "unknown command %s at line %d\n", fields[0], lnum);
	return(0);
}


void ja_processConfFile(char * pathconf)
{
	FILE *	fp;
	int	line;
	char 	buf[1024];


	fp = fopen(pathconf, "r");

	if (! fp)
	{
		fprintf(stderr, "cannot locate conf file [%s], aborting...\n", pathconf);
		exit(1);
	}

	for (line = 0;;line++)
	{
		if (!fgets(buf, 1000, fp))
			break;

		if (! strlen(buf))
			continue;

		buf[strlen(buf) - 1] = 0;

		if (processLine(buf, line) < 0)
			break;
	}

	fclose(fp);
}

