/*
 *
 * JONAMA - Secure TCP Relay with SSLeay
 *
 * ====================================================================
 * Copyright (c) 1999 Henri Gomez. All rights reserved.
 *
 * --------------------------------------------------------------------
 *
 * "This product includes software developed by
 * Ralf S. Engelschall <rse@engelschall.com> for use in the
 * mod_ssl project (http://www.engelschall.com/sw/mod_ssl/)."
 *
 * --------------------------------------------------------------------
 */

#include "jonama.h"

static char * conffile;


static void ja_init()
{
	jonama *	lja;
	int		i;


	for (i = 0, lja = ja_pool; i < MAX_JONAMA; i++, lja++)
	{
		lja->protocolLevel	= SSL_PROTOCOL_ALL;
		lja->context		= NULL;
		lja->connexion		= NULL;
		lja->verifyDepth	= 0;
		lja->verifyMode		= SSL_CVERIFY_NONE;
		lja->certificateFile	= NULL;
		lja->privateKeyFile	= NULL;
		lja->CAPath		= NULL;
		lja->CACertificateFile	= NULL;

		lja->acceptPort		= 8888;
		lja->numConnects	= 0;
		lja->connectMode	= C_MODE_NORMAL;
		lja->bufferSize		= S_BUFF_SIZE;
		lja->resolveRemote	= 0;
	}
}
	

static void ja_cleanup()
{
	int		i;
	jonama *	lja;

	ja_log(LOG_ALLWAYS, "Cleaning up services pool... (#%d)", numJa);

	for (i = 0, lja = ja_pool; i < numJa; i++, lja++)
		SSL_CTX_free(lja->context);
	
	numJa = 0;
}


static void ja_postinit(int first)
{
	char 	llogp[256];
	char	lerrp[256];


	sprintf(llogp, "%s/%s", logPath, logFileName);
	sprintf(lerrp, "%s/%s", logPath, errorFileName);
	ja_log_init(llogp, lerrp);
	ja_log(LOG_ALLWAYS, "Initializing...");

	if (first)
		ja_init_ssl();
}


static int ja_openbind(jonama * pja)
{
        struct sockaddr_in      listin;
	int			listensock;


        listensock = socket(AF_INET, SOCK_STREAM, 0);

        if (listensock < 0)
        {
                ja_logerr(LOG_ALLWAYS, "Unable to create socket.");
                return (-1);
        }

        memset(&listin, 0, sizeof(listin));
        listin.sin_family        = AF_INET;
        listin.sin_addr.s_addr  = htonl(INADDR_ANY);
        listin.sin_port         = htons(pja->acceptPort);

        if (bind(listensock, &listin, sizeof(listin)) < 0)
        {
                ja_logoserr(LOG_ALLWAYS, "Can't bind");
                return (-2);
        }

	return (listensock);
}


static void ja_run(int first)
{
	int		i;
	int		listensock;
	jonama *	lja;


	ja_init();
	ja_processConfFile(conffile);
	ja_postinit(first);

	if (first)
	{
		close(0);
		
		if (! log2stdx)
		{
			close(1);
			close(2);
		}
	}

	for (i = 0, lja = ja_pool; i < numJa; i++, lja++)
	{
		if (ja_new_context(lja) >= 0)
		{
			listensock = ja_openbind(lja);

			if (listensock >= 0)
				ja_server(lja, listensock);
		}
	}
}

static void usage()
{
	printf("usage: jonama args\n\n"
	       " -D             - daemonize\n"
	       " -f config.file - alternate config file\n"
	       " -s             - log to stderr instead of file\n"
	       "\n\n");

	exit(0);
}

static void ja_sgnhuphandler(int sig)
{
	switch (mode)
	{
		case T_MASTER :
		{
			ja_log(LOG_ALLWAYS, "Signal %d received, processing...", sig);
			ja_cleanup();
			ja_run(0);
			break;
		}

		case T_SERVER :	
		{
			ja_log(LOG_ALLWAYS, "Signal %d received, exiting...", sig);
			exit(1);
		}

		case T_CLIENT :
		{
			ja_log(LOG_ALLWAYS, "Signal %d received, ignoring...", sig);
			break;
		}

		default	     :
		{
			ja_log(LOG_ALLWAYS, "Signal %d received, aborting since mode not set (how ???)", sig);
			break;
		}

	}
}		
	
static void ja_processArgs(int argc, char **argv)
{
	int	i;


	for (i = 1; i < argc; i++)
	{
		if (argv[i][0] == '-')
		{
			switch (argv[i][1])
			{
				case 'D' :	daemonize = 1;	break;
				case 'f' :	
				{
					if (i < argc)
						conffile = argv[i+1];

					break;
				}

				case 's' :
				{
					log2stdx = 1;
					break;
				}

				default : usage();
			}
		}
	}
}


int main(int argc, char **argv)
{
	mode = T_MASTER;
	conffile = JONAMA_CONF_FILE;

	signal(SIGPIPE, SIG_IGN);
	signal(SIGCHLD, SIG_IGN);
	signal(SIGHUP,  ja_sgnhuphandler);

	ja_processArgs(argc, argv);

	if (daemonize)
	{
		switch (fork())
		{
			case -1 : fprintf(stderr, "cannot fork...\n");	exit(1);
			case  0 : break;
			default : exit(0);
		}
	}
	
	
	ja_run(1);

	for (;;)
		sleep(1);

	exit(0);
}

